package sample;

import java.util.HashMap;
import java.util.Map;

import com.ibm.task.api.ApplicationComponent;
import com.ibm.task.api.Escalation;
import com.ibm.task.api.EscalationTemplate;
import com.ibm.task.api.Task;
import com.ibm.task.api.TaskTemplate;
import com.ibm.task.api.WorkItem;
import com.ibm.task.spi.StaffQueryResult;
import com.ibm.task.spi.StaffQueryResultFactory;
import com.ibm.task.spi.StaffQueryResultPostProcessorPlugin;

/**
 * This class implements the logic of a Post Processor Plugin sample. It provides
 * for two work load balancing policies applied to the Potential Owner role of
 * tasks:
 * 
 * - Random user selection: selects one user from the set of received Potential
 * Owners for a given task
 * 
 * - Round Robin user selection: selects, from the set of received Potential
 * owners of a task, the one that follows to the last employed user.
 * 
 * The policy to be applied is retrieved from a custom property associated with
 * a given task.
 * 
 * If one the two policies is applicable, exactly one user is selected and
 * returned in the result. In all other cases, the original result is left
 * unchanged.
 * 
 */
public class PeopleSelectionPlugin implements
		StaffQueryResultPostProcessorPlugin {

	/**
	 * Enum definition for supported user selection policies.
	 */
	static enum UserSelectionPolicy {
		Random, RoundRobin
	}

	/*
	 * Method invoked for task roles. Logic is included for handling the
	 * Potential Owner role, only.
	 */
	public StaffQueryResult processStaffQueryResult(
			StaffQueryResult originalStaffQueryResult, Task task, int role,
			Map arg3) {
	    
	    if (task == null) {  // occurs during refresh 
	        System.out.println("PPPlugin: no processing during staff refresh");
	        return originalStaffQueryResult;
	    }

		// tracing
		System.out.println("PPPlugin: method for task: " + task.getName() + " role: " + getRoleAsString(role));

		// do not consider case for EVERYBODY, NOBODY, GROUPIDS result types
		if (originalStaffQueryResult.getResultType() != StaffQueryResult.RESULT_TYPE_USERIDS) {
			return originalStaffQueryResult;
		}

		// if nothing to be changed, retain result
		StaffQueryResult newStaffQueryResult = originalStaffQueryResult;

		// user selection applied to Potential Owner role, only
		// applies potentially to all task applications
		if (role == WorkItem.REASON_POTENTIAL_OWNER && task.getName() != null) {

			Map userDataMap = originalStaffQueryResult.getUserDataMap();
			String selectedUserId = null;

			// for no or one user, no selection is needed
			if (userDataMap.size() > 1) {
				UserSelectionPolicy policy = PolicyStore.getUserSelectionPolicyForTask(task);

				// tracing
				System.out.println("PPPlugin: user selection policy: " + policy);

				// if a policy was found, apply it
				if (policy != null) {
					switch (policy) {
					case Random:
						selectedUserId = RandomSelectionControl.getUserSelection(userDataMap);
						break;
					case RoundRobin:
						String taskKey = "{" + task.getDefinitionNamespace() + "}" + task.getDefinitionName();
						selectedUserId = RoundRobinControl.getUserSelection(userDataMap, taskKey);
						break;
					default:
						// nothing to do
					}
				}
			}

			// tracing
			System.out.println("PPPlugin: selectedUserId: " + selectedUserId);

			// compile new result, if a user was selected
			if (selectedUserId != null) {
				// create a new map
				Map newUserDataMap = new HashMap();

				// include the selected user along with its attributes from the
				// original map
				newUserDataMap.put(selectedUserId, userDataMap.get(selectedUserId));

				// create a new StaffQueryResult including the map
				newStaffQueryResult = StaffQueryResultFactory.newInstance().newStaffQueryResult(newUserDataMap);
			}
		}

		return newStaffQueryResult;
	}// public StaffQueryResult processStaffQueryResult(

	/*
	 * Method invoked for application component roles. No logic is included,
	 * i.e. the original StaffQueryResult is retained.
	 */
	public StaffQueryResult processStaffQueryResult(StaffQueryResult arg0,
			ApplicationComponent arg1, int arg2, Map arg3) {

		// tracing
		System.out.println("PPPlugin: method for app component: "
				+ arg1.getName() + " role: " + getRoleAsString(arg2));

		return arg0;
	}

	/*
	 * Method invoked for escalation template roles. No logic is included, i.e.
	 * the original StaffQueryResult is retained.
	 */
	public StaffQueryResult processStaffQueryResult(StaffQueryResult arg0,
			EscalationTemplate arg1, int arg2, Map arg3) {
		// tracing
		System.out.println("PPPlugin: method for esc template: "
				+ arg1.getName() + " role: " + getRoleAsString(arg2));

		return arg0;
	}

	/*
	 * Method invoked for task template roles. No logic is included, i.e. the
	 * original StaffQueryResult is retained.
	 */
	public StaffQueryResult processStaffQueryResult(StaffQueryResult arg0,
			TaskTemplate arg1, int arg2, Map arg3) {
		// tracing
		System.out.println("PPPlugin: method for task template: "
				+ arg1.getName() + " role: " + getRoleAsString(arg2));

		return arg0;
	}

	/*
	 * Method invoked for escalation roles. No logic is included, i.e. the
	 * original StaffQueryResult is retained.
	 */
	public StaffQueryResult processStaffQueryResult(StaffQueryResult arg0,
			Escalation arg1, Task arg2, int arg3, Map arg4) {
		// tracing
		System.out.println("PPPlugin: method for escalation: " + arg1.getName()
				+ " role: " + getRoleAsString(arg3));

		return arg0;
	}

	/**
	 * Helper method to retrieve textual names for the various roles.
	 * 
	 * @param role
	 *            Constant value (defined in the WorkItem class) indicating the
	 *            role.
	 * @return String representation of the role value.
	 */
	private String getRoleAsString(int role) {
		switch (role) {
		case WorkItem.REASON_ADMINISTRATOR:
			return "ADMINISTARTOR";
		case WorkItem.REASON_EDITOR:
			return "EDITOR";
		case WorkItem.REASON_ESCALATION_RECEIVER:
			return "ESCALATION_RECEIVER";
		case WorkItem.REASON_ORIGINATOR:
			return "ORIGINATOR";
		case WorkItem.REASON_OWNER:
			return "OWNER";
		case WorkItem.REASON_POTENTIAL_INSTANCE_CREATOR:
			return "POTENTIAL_INSTANCE_CREATOR";
		case WorkItem.REASON_POTENTIAL_OWNER:
			return "POTENTIAL_OWNER";
		case WorkItem.REASON_POTENTIAL_STARTER:
			return "POTENTIAL_STARTER";
		case WorkItem.REASON_READER:
			return "READER";
		case WorkItem.REASON_STARTER:
			return "STARTER";
		}

		return null;
	}
}
