package sample;

import java.util.Hashtable;

import sample.PeopleSelectionPlugin.UserSelectionPolicy;

import com.ibm.task.api.HumanTaskManager;
import com.ibm.task.api.Task;

/*
 *  Class providing a store for the user selection policy associated with
 *  tasks derived from the same task template. 
 *
 *  The store is realized as a static Hashtable and shared across plugin
 *  invocations.
 *  
 *  An entry in the tables consists of a (task key, user selection policy)
 *  pair, where the task key is derived from the name and namespace associated
 *  with tasks of the same task template.
 */
public class PolicyStore {
	private static Hashtable _taskPolicyTable = new Hashtable();
	private final static String policyCustomPropertyName = "userSelectionPolicy";

	/**
	 * Method to retrieve the possibly stored policy for a task. In case the
	 * policy is not stored for the task, the custom property
	 * "userSelectionPolicy" is retrieved via the HTM API.
	 * 
	 * If not set, no policy has been defined. If set, the value is matched
	 * against the supported values "Random", "RoundRobin". If either of these
	 * is found, the policy is returned as result, otherwise null is returned.
	 * 
	 * @param task
	 *            The task to examine.
	 * @return The policy supported.
	 */
	static UserSelectionPolicy getUserSelectionPolicyForTask(Task task) {
		// compute the task key from the task namespace and name
		String taskKey = "{" + task.getDefinitionNamespace() + "}"
				+ task.getDefinitionName();

		// check the store first for a stored policy
		UserSelectionPolicy policy = (UserSelectionPolicy) _taskPolicyTable
				.get(taskKey);

		// if no policy found in the store, retrieve it via the HTM API
		if (policy == null) {
			// get the HTM access object
			HumanTaskManager htm = BPCAccess.getHTM();

			// if HTM not available, no retrieval is possible
			if (htm != null) {
				String policyAsString = null;
				try {
					// get the custom property value
					policyAsString = htm.getCustomProperty(task.getID(),
							policyCustomPropertyName);
				} catch (Exception e) {
					// tracing
					System.out
							.println("PPPlugin: Could not retrieve custom property: "
									+ policyCustomPropertyName
									+ " for task: "
									+ taskKey);
				}

				// if no custom property value found, no policy can be
				// established
				if (policyAsString != null) {
					try {
						// match against supported policy values
						policy = UserSelectionPolicy.valueOf(policyAsString);

						// store the found value for subsequent tasks
						_taskPolicyTable.put(taskKey, policy);
					} catch (Exception e) {
						// tracing
						System.out
								.println("PPPlugin: Allocation policy with name: "
										+ policyAsString + " is not supported.");
					}

				}

			}
		}

		// return the found policy, or null, if none found
		return policy;
	}

}
