package sample;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.SortedSet;

import com.ibm.task.api.Task;

/**
 * Class implementing the RoundRobin user selection policy.
 * 
 */
public class RoundRobinControl {
	// Static store storing per task key, the last employed user.
	// A Hashtable is used due to its thread-safe implementation in Java.
	private static Hashtable _taskUserTable = new Hashtable();

	/**
	 * Method selecting the next user from the set of users contained in the
	 * map. The next user is computed using the last user Id employed for same
	 * task key.
	 * 
	 * A task key is computed from the namespace and name of the received task,
	 * which are assumed to be identical for all tasks derived from the same
	 * task template.
	 * 
	 * @param userDataMap
	 *            The received map containing the set of user Ids as keys.
	 * @param taskKey
	 *            The task key computed for the given task.
	 * @return The selected user Id.
	 */
	static String getUserSelection(Map userDataMap, String taskKey) {

		// get last selected user Id for the given task key
		String lastSelectedUserId = (String) _taskUserTable.get(taskKey);

		// sort received user IDs to have the same order within the set
		List userIds = new ArrayList(userDataMap.keySet());
		Collections.sort(userIds);

		// get index of last used userId
		int indexOfLastUser = -1;
		if (lastSelectedUserId != null) {
			indexOfLastUser = userIds.indexOf(lastSelectedUserId);
		}

		// select next user
		String selectedUserId = null;
		if ((indexOfLastUser > -1) && (indexOfLastUser < userIds.size() - 1)) {
			selectedUserId = (String) userIds.get(indexOfLastUser + 1);
		} else {
			selectedUserId = (String) userIds.get(0);
		}

		// update cache
		_taskUserTable.put(taskKey, selectedUserId);

		// return the selected user Id
		return selectedUserId;
	}

	// Local test method, user for testing purposes, only.
	// Not invoked during regular plugin execution.
	public static void main(String[] argv) {
		HashMap map = new HashMap();
		map.put("user1", "user1");
		map.put("user2", "user2");
		map.put("user3", "user3");
		map.put("user4", "user4");
		map.put("user5", "user5");
		map.put("user6", "user6");
		map.put("user7", "user7");

		String taskKey = "task1";
		_taskUserTable.put(taskKey, "user1");

		for (int i = 0; i < 10; i++) {
			System.out.println("round robin user: " + i + " is: "
					+ getUserSelection(map, taskKey));
		}
	}
}
